#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

#define LPC_USB_BASE				(0x40080000)
#define ENDPOINT_DETAILS_MAXEP			5
#define ENDPOINT_TOTAL_ENDPOINTS		ENDPOINT_DETAILS_MAXEP
#define ROMDRIVER_MEM_SIZE			0x500
#define USB_MAX_IF_NUM				8
#define USB_MAX_EP_NUM				6
#define USB_DEVICE_CLASS_HUMAN_INTERFACE	0x03
#define HID_REPORT_INPUT   			0x01
#define	HID_REPORT_OUTPUT			0x02
#define HID_REPORT_FEATURE			0x03
#define ENDPOINTSIZE				0x40

extern uint8_t *usb_RomDriver_buffer;

uint32_t CALLBACK_UsbdRom_Register_DeviceDescriptor(void);
uint32_t CALLBACK_UsbdRom_Register_ConfigurationDescriptor(void);
uint32_t CALLBACK_UsbdRom_Register_StringDescriptor(void);
uint32_t CALLBACK_UsbdRom_Register_DeviceQualifierDescriptor(void);
uint8_t  CALLBACK_UsbdRom_Register_ConfigureEndpoint(void);

typedef uint32_t USBD_HANDLE_T;
typedef void* USBD_HID_HANDLE_T;
typedef void* USB_CB_T;
typedef void* USB_PARAM_CB_T;

typedef enum { 
	LPC_OK=0,  ERR_FAILED = -1, 
	/* ISP related errors */
	ERR_ISP_BASE = 0x00000000,
	/*0x00000001*/ ERR_ISP_INVALID_COMMAND = ERR_ISP_BASE + 1,
	/*0x00000002*/ ERR_ISP_SRC_ADDR_ERROR, /* Source address not on word boundary */
	/*0x00000003*/ ERR_ISP_DST_ADDR_ERROR, /* Destination address not on word or 256 byte boundary */
	/*0x00000004*/ ERR_ISP_SRC_ADDR_NOT_MAPPED,
	/*0x00000005*/ ERR_ISP_DST_ADDR_NOT_MAPPED,
	/*0x00000006*/ ERR_ISP_COUNT_ERROR, /* Byte count is not multiple of 4 or is not a permitted value */
	/*0x00000007*/ ERR_ISP_INVALID_SECTOR,
	/*0x00000008*/ ERR_ISP_SECTOR_NOT_BLANK,
	/*0x00000009*/ ERR_ISP_SECTOR_NOT_PREPARED_FOR_WRITE_OPERATION,
	/*0x0000000A*/ ERR_ISP_COMPARE_ERROR,
	/*0x0000000B*/ ERR_ISP_BUSY, /* Flash programming hardware interface is busy */
	/*0x0000000C*/ ERR_ISP_PARAM_ERROR, /* Insufficient number of parameters */
	/*0x0000000D*/ ERR_ISP_ADDR_ERROR, /* Address not on word boundary */
	/*0x0000000E*/ ERR_ISP_ADDR_NOT_MAPPED,
	/*0x0000000F*/ ERR_ISP_CMD_LOCKED, /* Command is locked */
	/*0x00000010*/ ERR_ISP_INVALID_CODE, /* Unlock code is invalid */
	/*0x00000011*/ ERR_ISP_INVALID_BAUD_RATE,
	/*0x00000012*/ ERR_ISP_INVALID_STOP_BIT,
	/*0x00000013*/ ERR_ISP_CODE_READ_PROTECTION_ENABLED,

	/* ROM API related errors */
	ERR_API_BASE = 0x00010000, ERR_API_INVALID_PARAMS = ERR_API_BASE + 1,  ERR_API_INVALID_PARAM1,  ERR_API_INVALID_PARAM2,  ERR_API_INVALID_PARAM3,  ERR_API_MOD_INIT, 
	/* SPIFI API related errors */
	ERR_SPIFI_BASE = 0x00020000,
	/*0x00020001*/ ERR_SPIFI_DEVICE_ERROR =ERR_SPIFI_BASE+1,
	/*0x00020002*/ ERR_SPIFI_INTERNAL_ERROR,
	/*0x00020003*/ ERR_SPIFI_TIMEOUT,
	/*0x00020004*/ ERR_SPIFI_OPERAND_ERROR,
	/*0x00020005*/ ERR_SPIFI_STATUS_PROBLEM,
	/*0x00020006*/ ERR_SPIFI_UNKNOWN_EXT,
	/*0x00020007*/ ERR_SPIFI_UNKNOWN_ID,
	/*0x00020008*/ ERR_SPIFI_UNKNOWN_TYPE,
	/*0x00020009*/ ERR_SPIFI_UNKNOWN_MFG,

	/* Security API related errors */
	ERR_SEC_BASE = 0x00030000,
	/*0x00030001*/ ERR_SEC_AES_WRONG_CMD=ERR_SEC_BASE+1,
	/*0x00030002*/ ERR_SEC_AES_NOT_SUPPORTED,
	/*0x00030003*/ ERR_SEC_AES_KEY_ALREADY_PROGRAMMED,

	/* USB device stack related errors */
	ERR_USBD_BASE = 0x00040000, ERR_USBD_INVALID_REQ = ERR_USBD_BASE + 1,  ERR_USBD_UNHANDLED,  ERR_USBD_STALL,      ERR_USBD_SEND_ZLP,   ERR_USBD_SEND_DATA,  ERR_USBD_BAD_DESC,   ERR_USBD_BAD_CFG_DESC, ERR_USBD_BAD_INTF_DESC, ERR_USBD_BAD_EP_DESC, ERR_USBD_BAD_MEM_BUF,  ERR_USBD_TOO_MANY_CLASS_HDLR, 
	/* CGU  related errors */
	ERR_CGU_BASE = 0x00050000,
	/*0x00050001*/ ERR_CGU_NOT_IMPL=ERR_CGU_BASE+1,
	/*0x00050002*/ ERR_CGU_INVALID_PARAM,
	/*0x00050003*/ ERR_CGU_INVALID_SLICE,
	/*0x00050004*/ ERR_CGU_OUTPUT_GEN,
	/*0x00050005*/ ERR_CGU_DIV_SRC,
	/*0x00050006*/ ERR_CGU_DIV_VAL,
	/*0x00050007*/ ERR_CGU_SRC

} ErrorCode_t;

enum  	USBD_EVENT_T {
  USB_EVT_SETUP = 1, USB_EVT_OUT, USB_EVT_IN, USB_EVT_OUT_NAK,
  USB_EVT_IN_NAK, USB_EVT_OUT_STALL, USB_EVT_IN_STALL, USB_EVT_OUT_DMA_EOT,
  USB_EVT_IN_DMA_EOT, USB_EVT_OUT_DMA_NDR, USB_EVT_IN_DMA_NDR, USB_EVT_OUT_DMA_ERR,
  USB_EVT_IN_DMA_ERR, USB_EVT_RESET, USB_EVT_SOF, USB_EVT_DEV_STATE,
  USB_EVT_DEV_ERROR
};

typedef ErrorCode_t (*USB_EP_HANDLER_T)(USBD_HANDLE_T hUsb, void* data, uint32_t event);

typedef struct _USB_EP_DATA
{
	uint8_t *pData;
	uint16_t Count;
} USB_EP_DATA;

typedef struct _HID_REPORT_T {
	uint16_t len; /**< Size of the report descriptor in bytes. */ 
	uint8_t idle_time; /**< This value is used by stack to respond to Set_Idle & 
				GET_Idle requests for the specified report ID. The value
				of this field specified the rate at which duplicate reports 
				are generated for the specified Report ID. For example, a 
				device with two input reports could specify an idle rate of 
				20 milliseconds for report ID 1 and 500 milliseconds for 
				report ID 2.
			   */
	uint8_t __pad; 	   /**< Padding space. */
	uint8_t* desc;	   /**< Report descriptor. */
} USB_HID_REPORT_T;

typedef struct _USB_REPORT {
	uint8_t	length;
	uint8_t data[64];
} USB_REPORT;

typedef ErrorCode_t (*USB_EP_HANDLER_T)(USBD_HANDLE_T hUsb, void* data, uint32_t event);

struct __attribute__((__packed__)) _WB_T {
	uint8_t L; 
	uint8_t H; 
};
typedef struct _WB_T WB_T;

union __attribute__((__packed__)) __WORD_BYTE {
	uint16_t W; 
	WB_T WB; 
};
typedef union __WORD_BYTE WORD_BYTE;

struct __attribute__((__packed__)) _BM_T {
	uint8_t Recipient :  5; 
	uint8_t Type      :  2; 
	uint8_t Dir       :  1; 
};
typedef struct _BM_T BM_T;

union __attribute__((__packed__)) _REQUEST_TYPE {
	uint8_t B; 
	BM_T BM;   
};
typedef union _REQUEST_TYPE REQUEST_TYPE;

struct __attribute__((__packed__)) _CDC_LINE_CODING {
	uint32_t dwDTERate;                            /* Data terminal rate in bits per second */
	uint8_t  bCharFormat;                          /* Number of stop bits */
	uint8_t  bParityType;                          /* Parity bit type */
	uint8_t  bDataBits;                            /* Number of data bits */
};
typedef struct _CDC_LINE_CODING CDC_LINE_CODING;

typedef struct _USB_CORE_DESCS_T {
	uint8_t *device_desc; 		/**< Pointer to USB device descriptor */
	uint8_t *string_desc; 		/**< Pointer to array of USB string descriptors */
	uint8_t *full_speed_desc; 	/**< Pointer to USB device configuration descriptor
					 * when device is operating in full speed mode.
					 */
	uint8_t *high_speed_desc;	/**< Pointer to USB device configuration descriptor
					 * when device is operating in high speed mode. For
					 * full-speed only implementation this pointer should
					 * be same as full_speed_desc.
					 */
	uint8_t *device_qualifier;	/**< Pointer to USB device qualifier descriptor. For
					 * full-speed only implementation this pointer should
					 * be set to null (0).
					 */
} USB_CORE_DESCS_T;

struct __attribute__((__packed__)) _USB_ENDPOINT_DESCRIPTOR {
	uint8_t  bLength; 
	uint8_t  bDescriptorType; 
	uint8_t  bEndpointAddress; 
	uint8_t  bmAttributes; 
	uint16_t wMaxPacketSize; 
	uint8_t  bInterval; 
};
typedef struct _USB_ENDPOINT_DESCRIPTOR USB_ENDPOINT_DESCRIPTOR;

typedef struct USBD_API_INIT_PARAM {
	uint32_t usb_reg_base; 
	uint32_t mem_base;  
	uint32_t mem_size;  
	uint8_t max_num_ep; 
	uint8_t pad0[3];

	/* USB Device Events Callback Functions */
	USB_CB_T USB_Reset_Event;
	USB_CB_T USB_Suspend_Event;
	USB_CB_T USB_Resume_Event;
	USB_CB_T reserved_sbz;
	USB_CB_T USB_SOF_Event;
	USB_PARAM_CB_T USB_WakeUpCfg;
	USB_PARAM_CB_T USB_Power_Event;
	USB_PARAM_CB_T USB_Error_Event;

	/* USB Core Events Callback Functions */
	USB_CB_T USB_Configure_Event;
	USB_CB_T USB_Interface_Event;
	USB_CB_T USB_Feature_Event;
	
	/* cache and MMU translation functions */
	uint32_t (* virt_to_phys)(void* vaddr);
	void (* cache_flush)(uint32_t* start_adr, uint32_t* end_adr);
} USBD_API_INIT_PARAM_T;

typedef struct USBD_MSC_INIT_PARAM {
	/* memory allocation params */
	uint32_t mem_base;  	/**< Base memory location from where the stack can allocate
			             data and buffers. @note The memory address set in this field
			       	     should be accessible by USB DMA controller. Also this value
			       	     should be aligned on 4 byte boundary.
			    	*/
	uint32_t mem_size;  	/**< The size of memory buffer which stack can use.
			       	     @note The \em mem_size should be greater than the size
			       	     returned by USBD_MSC_API::GetMemSize() routine.
				*/

	/* mass storage paramas */
	uint8_t* InquiryStr;	/**< Pointer to the 28 character string. This string is
				     sent in response to the SCSI Inquiry command. @note The data
				     pointed by the pointer should be of global scope.
				*/
	uint32_t BlockCount;	/**< Number of blocks present in the mass storage device */
	uint32_t BlockSize; 	/**< Block size in number of bytes */
	uint32_t MemorySize; 	/**< Memory size in number of bytes */

	/** Pointer to the interface descriptor within the descriptor
	  * array (\em high_speed_desc) passed to Init() through @ref USB_CORE_DESCS_T
	  * structure. The stack assumes both HS and FS use same BULK endpoints.
	  */
	uint8_t* intf_desc;

	/* user defined functions */

	/**
	  *  MSC Write callback function.
	  *
	  *  This function is provided by the application software. This function gets called
	  *  when host sends a write command.
	  *
	  *  @param offset Destination start address.
	  *  @param src  Pointer to a pointer to the source of data. Pointer-to-pointer
	  *                       is used to implement zero-copy buffers.
	  *  @param length  Number of bytes to be written.
	  *  @return Nothing.
	  *
	  */

	void (*MSC_Write)( uint32_t offset, uint8_t** src, uint32_t length);

	/**
	  *  MSC Read callback function.
	  *
	  *  This function is provided by the application software. This function gets called
	  *  when host sends a read command.
	  *
	  *  @param offset Source start address.
	  *  @param dst  Pointer to a pointer to the source of data. The MSC function drivers
	  *         implemented in stack are written with zero-copy model. Meaning the stack doesn't make an	
	  *          extra copy of buffer before writing/reading data from USB hardware FIFO. Hence the
	  *          parameter is pointer to a pointer containing address buffer (<em>uint8_t** dst</em>).
	  *          So that the user application can update the buffer pointer instead of copying data to
	  *          address pointed by the parameter. /note The updated buffer address should be accessable
	  *          by USB DMA master. If user doesn't want to use zero-copy model, then the user should copy
	  *          data to the address pointed by the passed buffer pointer parameter and shouldn't change
	  *          the address value.
	  *  @param length  Number of bytes to be read.
	  *  @return Nothing.
	  *
	  */

	void (*MSC_Read)( uint32_t offset, uint8_t** dst, uint32_t length);

	 /**
	  *  MSC Verify callback function.
	  *
	  *  This function is provided by the application software. This function gets called
	  *  when host sends a verify command. The callback function should compare the buffer
	  *  with the destination memory at the requested offset and
	  *
	  *  @param offset Destination start address.
	  *  @param buf  Buffer containing the data sent by the host.
	  *  @param length  Number of bytes to verify.
	  *  @return Returns @ref ErrorCode_t type to indicate success or error condition.
	  *          @retval LPC_OK If data in the buffer matches the data at destination
	  *          @retval ERR_FAILED  Atleast one byte is different.
	  *
	  */

	ErrorCode_t (*MSC_Verify)( uint32_t offset, uint8_t buf[], uint32_t length);

	/**
	  *  Optional callback function to optimize MSC_Write buffer transfer.	
	  *
	  *  This function is provided by the application software. This function gets called
	  *  when host sends SCSI_WRITE10/SCSI_WRITE12 command. The callback function should
	  *  update the \em buff_adr pointer so that the stack transfers the data directly
	  *  to the target buffer. /note The updated buffer address should be accessable
	  *  by USB DMA master. If user doesn't want to use zero-copy model, then the user
	  *  should not update the buffer pointer.
	  *
	  *  @param offset Destination start address.
	  *  \param[in,out] buf  Buffer containing the data sent by the host.
	  *  @param length  Number of bytes to write.
	  *  @return Nothing.
	  *
	  */

	void (*MSC_GetWriteBuf)( uint32_t offset, uint8_t** buff_adr, uint32_t length);

	/**
	*  Optional user overridable function to replace the default MSC class handler.
	*
	*  The application software could override the default EP0 class handler with their
	*  own by providing the handler function address as this data member of the parameter
	*  structure. Application which like the default handler should set this data member
	*  to zero before calling the USBD_MSC_API::Init().
	*  \n
	*  @note
	*
	*  @param hUsb Handle to the USB device stack.
	*  @param data Pointer to the data which will be passed when callback function is called by the stack.
	*  @param event  Type of endpoint event. See @ref USBD_EVENT_T for more details.
	*  @return The call back should returns @ref ErrorCode_t type to indicate success or error condition.
	*          @retval LPC_OK On success.
	*          @retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line.
	*          @retval ERR_USBD_xxx  For other error conditions.
	*
	*/

	ErrorCode_t (*MSC_Ep0_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

} USBD_MSC_INIT_PARAM_T;

typedef struct USBD_DFU_INIT_PARAM {

	/* memory allocation params */

	uint32_t mem_base;  
	uint32_t mem_size;  

	/* DFU paramas */

	uint16_t wTransferSize; 
	uint16_t pad;
	uint8_t* intf_desc;

	/* user defined functions */

	uint8_t (*DFU_Write)( uint32_t block_num, uint8_t** src, uint32_t length, uint8_t* bwPollTimeout);
	uint32_t (*DFU_Read)( uint32_t block_num, uint8_t** dst, uint32_t length);
	void (*DFU_Done)(void);
	void (*DFU_Detach)(USBD_HANDLE_T hUsb);
	ErrorCode_t (*DFU_Ep0_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);
} USBD_DFU_INIT_PARAM_T;

struct __attribute__((__packed__)) _USB_SETUP_PACKET {
	REQUEST_TYPE bmRequestType; /**< This bitmapped field identifies the characteristics
					 of the specific request. \sa _BM_T.
				    */
	uint8_t		bRequest;   /**< This field specifies the particular request. The
					 Type bits in the bmRequestType field modify the meaning
					 of this field. \sa USBD_REQUEST.
				    */
	WORD_BYTE	wValue;	    /**< Used to pass a parameter to the device, specific
					 to the request.
				    */
	WORD_BYTE	wIndex;	    /**< Used to pass a parameter to the device, specific
					 to the request. The wIndex field is often used in
					 requests to specify an endpoint or an interface.
				    */
	uint16_t	wLength;    /**< This field specifies the length of the data
					 transferred during the second phase of the control
					 transfer.
				    */
};
typedef struct _USB_SETUP_PACKET USB_SETUP_PACKET;

typedef struct __USBD_HID_INIT_PARAM_T {
	/* memory allocation params */
  	uint32_t  mem_base;
	uint32_t  mem_size;

	/* HID paramas */
	uint8_t max_reports;

	/* Pointer to the interface descriptor within the descriptor
	 * array passed to USBD_Init().  
	 */
	uint8_t* intf_desc;
	USB_HID_REPORT_T* report_data;

	/* user defined functions */
	/* required functions */

	ErrorCode_t (*HID_GetReport)( USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t* length); 
	ErrorCode_t (*HID_SetReport)( USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t length);

	/* optional functions */

	ErrorCode_t (*HID_GetPhysDesc)( USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuf, uint16_t* length);
	ErrorCode_t (*HID_SetIdle)( USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t idleTime); 
	ErrorCode_t (*HID_SetProtocol)( USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t protocol); 
	ErrorCode_t (*HID_EpIn_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);
	ErrorCode_t (*HID_EpOut_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

	/* user overridable function */

	ErrorCode_t (*HID_GetReportDesc)(USBD_HID_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuf, uint16_t* length);
	ErrorCode_t (*HID_Ep0_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

} USBD_HID_INIT_PARAM_T;

typedef struct USBD_CDC_INIT_PARAM {
	/* memory allocation params */

	uint32_t mem_base;  /**< Base memory location from where the stack can allocate
				 data and buffers. \note The memory address set in this field
				 should be accessible by USB DMA controller. Also this value
				 should be aligned on 4 byte boundary.
			    */
	uint32_t mem_size;  /**< The size of memory buffer which stack can use. 
				 \note The \em mem_size should be greater than the size 
				 returned by USBD_CDC_API::GetMemSize() routine.
			    */

	/** Pointer to the control interface descriptor within the descriptor
	  * array (\em high_speed_desc) passed to Init() through \ref USB_CORE_DESCS_T 
	  * structure. The stack assumes both HS and FS use same BULK endpoints. 
	  */

	uint8_t* cif_intf_desc;

	/** Pointer to the data interface descriptor within the descriptor
	  * array (\em high_speed_desc) passed to Init() through \ref USB_CORE_DESCS_T 
	  * structure. The stack assumes both HS and FS use same BULK endpoints. 
	  */

	uint8_t* dif_intf_desc;

	/* user defined functions */

	/* required functions */
	/** 
	  *  Communication Interface Class specific get request callback function.
	  *
	  *  This function is provided by the application software. This function gets called 
	  *  when host sends CIC management element get requests. The setup packet data (\em pSetup)
	  *  is passed to the callback so that application can extract the CIC request type
	  *  and other associated data. By default the stack will ssign \em pBuffer pointer
	  *  to \em EP0Buff allocated at init. The application code can directly write data 
	  *  into this buffer as long as data is less than 64 byte. If more data has to be sent 
	  *  then application code should update \em pBuffer pointer and length accordingly.
	  *   
	  *  
	  *  \param[in] hCdc Handle to CDC function driver. 
	  *  \param[in] pSetup Pointer to setup packet recived from host. 
	  *  \param[in, out] pBuffer  Pointer to a pointer of data buffer containing request data. 
	  *                       Pointer-to-pointer is used to implement zero-copy buffers. 
	  *                       See \ref USBD_ZeroCopy for more details on zero-copy concept.
	  *  \param[in, out] length  Amount of data to be sent back to host.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CIC_GetRequest)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t* length); 
  
	/** 
	  *  Communication Interface Class specific set request callback function.
	  *
	  *  This function is provided by the application software. This function gets called 
	  *  when host sends a CIC management element requests. The setup packet data (\em pSetup)
	  *  is passed to the callback so that application can extract the CIC request type
	  *  and other associated data. If a set request has data associated, then this callback
	  *  is called twice. 
	  *  (1) First when setup request is recived, at this time application code could update
	  *  \em pBuffer pointer to point to the intended destination. The length param is set to 0
	  *  so that application code knows this is first time. By default the stack will
	  *  assign \em pBuffer pointer to \em EP0Buff allocated at init. Note, if data length is 
	  *  greater than 64 bytes and application code doesn't update \em pBuffer pointer the 
	  *  stack will send STALL condition to host.
	  *  (2) Second when the data is recived from the host. This time the length param is set
	  *  with number of data bytes recived.
	  *  
	  *  \param[in] hCdc Handle to CDC function driver. 
	  *  \param[in] pSetup Pointer to setup packet recived from host. 
	  *  \param[in, out] pBuffer  Pointer to a pointer of data buffer containing request data. 
	  *                       Pointer-to-pointer is used to implement zero-copy buffers. 
	  *                       See \ref USBD_ZeroCopy for more details on zero-copy concept.
	  *  \param[in] length  Amount of data copied to destination buffer.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CIC_SetRequest)( USBD_HANDLE_T hCdc, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t length);

	/** 
	  *  Communication Device Class specific BULK IN endpoint handler.
	  *
	  *  The application software should provide the BULK IN endpoint handler.
	  *  Applications should transfer data depending on the communication protocol type set in descriptors. 
	  *  \n
	  *  \note 
	  *  
	  *  \param[in] hUsb Handle to the USB device stack. 
	  *  \param[in] data Pointer to the data which will be passed when callback function is called by the stack. 
	  *  \param[in] event  Type of endpoint event. See \ref USBD_EVENT_T for more details.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CDC_BulkIN_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

	/** 
	  *  Communication Device Class specific BULK OUT endpoint handler.
	  *
	  *  The application software should provide the BULK OUT endpoint handler.
	  *  Applications should transfer data depending on the communication protocol type set in descriptors. 
	  *  \n
	  *  \note 
	  *  
	  *  \param[in] hUsb Handle to the USB device stack. 
	  *  \param[in] data Pointer to the data which will be passed when callback function is called by the stack. 
	  *  \param[in] event  Type of endpoint event. See \ref USBD_EVENT_T for more details.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CDC_BulkOUT_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);
  	ErrorCode_t (*SendEncpsCmd) (USBD_HANDLE_T hCDC, uint8_t* buffer, uint16_t len);
	ErrorCode_t (*GetEncpsResp) (USBD_HANDLE_T hCDC, uint8_t** buffer, uint16_t* len);
	ErrorCode_t (*SetCommFeature) (USBD_HANDLE_T hCDC, uint16_t feature, uint8_t* buffer, uint16_t len);
	ErrorCode_t (*GetCommFeature) (USBD_HANDLE_T hCDC, uint16_t feature, uint8_t** pBuffer, uint16_t* len);
	ErrorCode_t (*ClrCommFeature) (USBD_HANDLE_T hCDC, uint16_t feature);
	ErrorCode_t (*SetCtrlLineState) (USBD_HANDLE_T hCDC, uint16_t state);
	ErrorCode_t (*SendBreak) (USBD_HANDLE_T hCDC, uint16_t mstime);
	ErrorCode_t (*SetLineCode) (USBD_HANDLE_T hCDC, CDC_LINE_CODING* line_coding);

	/** 
	  *  Optional Communication Device Class specific INTERRUPT IN endpoint handler.
	  *
	  *  The application software should provide the INT IN endpoint handler.
	  *  Applications should transfer data depending on the communication protocol type set in descriptors. 
	  *  \n
	  *  \note 
	  *  
	  *  \param[in] hUsb Handle to the USB device stack. 
	  *  \param[in] data Pointer to the data which will be passed when callback function is called by the stack. 
	  *  \param[in] event  Type of endpoint event. See \ref USBD_EVENT_T for more details.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CDC_InterruptEP_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

	/** 
	  *  Optional user overridable function to replace the default CDC class handler.
	  *
	  *  The application software could override the default EP0 class handler with their
	  *  own by providing the handler function address as this data member of the parameter
	  *  structure. Application which like the default handler should set this data member
	  *  to zero before calling the USBD_CDC_API::Init().
	  *  \n
	  *  \note 
	  *  
	  *  \param[in] hUsb Handle to the USB device stack. 
	  *  \param[in] data Pointer to the data which will be passed when callback function is called by the stack. 
	  *  \param[in] event  Type of endpoint event. See \ref USBD_EVENT_T for more details.
	  *  \return The call back should returns \ref ErrorCode_t type to indicate success or error condition.
	  *          \retval LPC_OK On success.
	  *          \retval ERR_USBD_UNHANDLED  Event is not handled hence pass the event to next in line. 
	  *          \retval ERR_USBD_xxx  For other error conditions. 
	  *                                             
	  */

	ErrorCode_t (*CDC_Ep0_Hdlr) (USBD_HANDLE_T hUsb, void* data, uint32_t event);

} USBD_CDC_INIT_PARAM_T;

typedef struct USBD_HW_API_T {
	uint32_t(*GetMemSize)(USBD_API_INIT_PARAM_T *param);
	ErrorCode_t(*Init)(USBD_HANDLE_T *phUsb, USB_CORE_DESCS_T *pDesc, USBD_API_INIT_PARAM_T *param);
	void(*Connect)(USBD_HANDLE_T hUsb, uint32_t con);
	void(*ISR)(USBD_HANDLE_T hUsb);
	void(*Reset)(USBD_HANDLE_T hUsb);
	void(*ForceFullSpeed)(USBD_HANDLE_T hUsb, uint32_t cfg);
	void(*WakeUpCfg)(USBD_HANDLE_T hUsb, uint32_t cfg);
	void(*SetAddress)(USBD_HANDLE_T hUsb, uint32_t adr);
	void(*Configure)(USBD_HANDLE_T hUsb, uint32_t cfg);
	void(*ConfigEP)(USBD_HANDLE_T hUsb, USB_ENDPOINT_DESCRIPTOR *pEPD);
	void(*DirCtrlEP)(USBD_HANDLE_T hUsb, uint32_t dir);
	void(*EnableEP)(USBD_HANDLE_T hUsb, uint32_t EPNum);
	void(*DisableEP)(USBD_HANDLE_T hUsb, uint32_t EPNum);
	void(*ResetEP)(USBD_HANDLE_T hUsb, uint32_t EPNum);
	void(*SetStallEP)(USBD_HANDLE_T hUsb, uint32_t EPNum);
	void(*ClrStallEP)(USBD_HANDLE_T hUsb, uint32_t EPNum);
	ErrorCode_t(*SetTestMode)(USBD_HANDLE_T hUsb, uint8_t mode);
	uint32_t(*ReadEP)(USBD_HANDLE_T hUsb, uint32_t EPNum, uint8_t *pData);
	uint32_t(*ReadReqEP)(USBD_HANDLE_T hUsb, uint32_t EPNum, uint8_t *pData, uint32_t len);
	uint32_t(*ReadSetupPkt)(USBD_HANDLE_T hUsb, uint32_t EPNum, uint32_t *pData);
	uint32_t(*WriteEP)(USBD_HANDLE_T hUsb, uint32_t EPNum, uint8_t *pData, uint32_t cnt);
	void(*WakeUp)(USBD_HANDLE_T hUsb);
	ErrorCode_t(*EnableEvent)(USBD_HANDLE_T hUsb, uint32_t EPNum, uint32_t event_type, uint32_t enable);
} USBD_HW_API_T;

typedef struct USBD_CORE_API_T {
	ErrorCode_t(*RegisterClassHandler )(USBD_HANDLE_T hUsb, USB_EP_HANDLER_T pfn, void *data);
	ErrorCode_t(*RegisterEpHandler )(USBD_HANDLE_T hUsb, uint32_t ep_index, USB_EP_HANDLER_T pfn, void *data);
	void(*SetupStage)(USBD_HANDLE_T hUsb);
	void(*DataInStage)(USBD_HANDLE_T hUsb);
	void(*DataOutStage)(USBD_HANDLE_T hUsb);
	void(*StatusInStage)(USBD_HANDLE_T hUsb);
	void(*StatusOutStage)(USBD_HANDLE_T hUsb);
	void(*StallEp0)(USBD_HANDLE_T hUsb);
} USBD_CORE_API_T;

typedef struct USBD_MSC_API_T {
	uint32_t(*GetMemSize)(USBD_MSC_INIT_PARAM_T *param);
	ErrorCode_t(*init)(USBD_HANDLE_T hUsb, USBD_MSC_INIT_PARAM_T *param);
} USBD_MSC_API_T;

typedef struct USBD_DFU_API_T {
	uint32_t(*GetMemSize)(USBD_DFU_INIT_PARAM_T *param);
	ErrorCode_t(*init)(USBD_HANDLE_T hUsb, USBD_DFU_INIT_PARAM_T *param, uint32_t init_state);
} USBD_DFU_API_T;

typedef struct USBD_HID_API_T {
	uint32_t(*GetMemSize)(USBD_HID_INIT_PARAM_T *param);
	ErrorCode_t(*init)(USBD_HANDLE_T hUsb, USBD_HID_INIT_PARAM_T *param);
} USBD_HID_API_T;

typedef struct USBD_CDC_API_T {
	uint32_t(*GetMemSize)(USBD_CDC_INIT_PARAM_T *param);
	ErrorCode_t(*init)(USBD_HANDLE_T hUsb, USBD_CDC_INIT_PARAM_T *param, USBD_HANDLE_T *phCDC);
	ErrorCode_t(*SendNotification)(USBD_HANDLE_T hCdc, uint8_t bNotification, uint16_t data);	
} USBD_CDC_API_T;
    
typedef struct USBD_API_T {
	const USBD_HW_API_T*	hw;
	const USBD_CORE_API_T*	core;
	const USBD_MSC_API_T*	msc;
	const USBD_DFU_API_T*	dfu;
	const USBD_HID_API_T* 	hid;
	const USBD_CDC_API_T*	cdc;
	const uint32_t*		reserved6;
	const uint32_t		version;
} USBD_API_T;

typedef	struct _ROM {
	const unsigned p_usbd;
	const unsigned p_clib;
	const unsigned p_cand;
#ifdef PWRROMD_PRESENT
	const PWRD * pPWRD;
#else
	const unsigned p_pwrd;
#endif /* PWRROMD_PRESENT */
#ifdef DIVROMD_PRESENT
	const LPC_ROM_DIV_STRUCT* pROMDiv;
#else
	const unsigned p_dev1;
#endif /* DIVROMD_PRESENT */
	const unsigned p_dev2;
	const unsigned p_dev3;
	const unsigned p_dev4;
}  ROM_FUNCTION_TABLE;

#define USBD_API 			(((USBD_API_T*)(ROM_USBD_PTR)))
#define ROM_FUNCTION_TABLE_PTR_ADDR	(0x1FFF1FF8UL)
#define ROM_USBD_PTR 			((*(ROM_FUNCTION_TABLE **)(ROM_FUNCTION_TABLE_PTR_ADDR))->p_usbd)

struct _USB_CORE_CTRL_T {
	/* overridable function pointers ~ c++ style virtual functions*/

	USB_CB_T USB_EvtSetupHandler;
	USB_CB_T USB_EvtOutHandler;
	USB_PARAM_CB_T USB_ReqVendor;
	USB_CB_T USB_ReqGetStatus;
	USB_CB_T USB_ReqGetDescriptor;
	USB_CB_T USB_ReqGetConfiguration;
	USB_CB_T USB_ReqSetConfiguration;
	USB_CB_T USB_ReqGetInterface;
	USB_CB_T USB_ReqSetInterface;
	USB_PARAM_CB_T USB_ReqSetClrFeature;

	/* USB Device Events Callback Functions */

	USB_CB_T USB_Reset_Event;
	USB_CB_T USB_Suspend_Event;
	USB_CB_T USB_Resume_Event;
	USB_CB_T USB_SOF_Event;
	USB_PARAM_CB_T USB_Power_Event;
	USB_PARAM_CB_T USB_Error_Event;
	USB_PARAM_CB_T USB_WakeUpCfg;

	/* USB Core Events Callback Functions */

	USB_CB_T USB_Configure_Event;
	USB_CB_T USB_Interface_Event;
	USB_CB_T USB_Feature_Event;

	/* cache and mmu translation functions */

	uint32_t (* virt_to_phys)(void* vaddr);
	void (* cache_flush)(uint32_t* start_adr, uint32_t* end_adr);

	/* event handlers for endpoints. */

	USB_EP_HANDLER_T  ep_event_hdlr[2 * USB_MAX_EP_NUM];
	void*  ep_hdlr_data[2 * USB_MAX_EP_NUM];

	/* USB class handlers */

	USB_EP_HANDLER_T  ep0_hdlr_cb[USB_MAX_IF_NUM];
	void*  ep0_cb_data[USB_MAX_IF_NUM];
	uint8_t num_ep0_hdlrs;

	/* USB Core data Variables */

	uint8_t max_num_ep; /* max number of endpoints supported by the HW */
	uint8_t device_speed;
	uint8_t  num_interfaces;
	uint8_t  device_addr;
	uint8_t  config_value;
	uint16_t device_status;
	uint8_t *device_desc;
	uint8_t *string_desc;
	uint8_t *full_speed_desc;
	uint8_t *high_speed_desc;
	uint8_t *device_qualifier;
	uint32_t ep_mask;
	uint32_t ep_halt;
	uint32_t ep_stall;
	uint8_t  alt_setting[USB_MAX_IF_NUM];

	/* HW driver data pointer */

	void* hw_data;

	/* USB Endpoint 0 Data Info */

	USB_EP_DATA EP0Data;

	/* USB Endpoint 0 Buffer */
	//ALIGNED(4)
	uint8_t  EP0Buf[64];

	/* USB Setup Packet */
	//ALIGNED(4)
	USB_SETUP_PACKET SetupPacket;
};
typedef struct _USB_CORE_CTRL_T  USB_CORE_CTRL_T;

typedef struct _HID_CTRL_T {
	/* pointer to controller */
	USB_CORE_CTRL_T*  pUsbCtrl;
	/* descriptor pointers */
	uint8_t* hid_desc;
	USB_HID_REPORT_T* report_data;

	uint8_t protocol;
	uint8_t if_num;                  /* interface number */
	uint8_t epin_adr;                /* IN interrupt endpoint */
	uint8_t epout_adr;               /* OUT interrupt endpoint */

	/* user defined functions */
	ErrorCode_t (*HID_GetReport)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t* length);
	ErrorCode_t (*HID_SetReport)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t length);
	ErrorCode_t (*HID_GetPhysDesc)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuf, uint16_t* length);
	ErrorCode_t (*HID_SetIdle)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t idleTime);
	ErrorCode_t (*HID_SetProtocol)( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t protocol);

	/* virtual overridable functions */
	ErrorCode_t (*HID_GetReportDesc)(USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuf, uint16_t* length);

}USB_HID_CTRL_T;

struct __attribute__((__packed__)) _USB_INTERFACE_DESCRIPTOR
{
	uint8_t  bLength; 
	uint8_t  bDescriptorType; 
	uint8_t  bInterfaceNumber; 
	uint8_t  bAlternateSetting; 
	uint8_t  bNumEndpoints; 
	uint8_t  bInterfaceClass; 
	uint8_t  bInterfaceSubClass; 
	uint8_t  bInterfaceProtocol; 
	uint8_t  iInterface; 
};
typedef struct _USB_INTERFACE_DESCRIPTOR USB_INTERFACE_DESCRIPTOR;

void UsbdRom_Init(USBD_API_INIT_PARAM_T* usb_param);
void USB_IRQHandler(void);
void EnableEP(USBD_HANDLE_T hUsb, uint32_t EPnum);
void DisableEP(USBD_HANDLE_T hUsb, uint32_t EPnum);
void USBconnect(void);
void USBdisconnect(void);

ErrorCode_t usb_hid_init(USBD_API_INIT_PARAM_T* usb_param);
ErrorCode_t EP0_patch(USBD_HANDLE_T hUsb, void *data, uint32_t event);
