#include "lpc11u24.h"
#include "usb.h"
#include "descriptors.h"
#include <string.h>

uint8_t *usb_RomDriver_buffer = (uint8_t*) 0x20004000;

USBD_HANDLE_T UsbHandle;
USB_REPORT *Report;

uint32_t g_ep0RxBusy = 0;
USB_EP_HANDLER_T g_Ep0BaseHdlr;
uint8_t tx_busy=0;
/*
ErrorCode_t EP0_patch(USBD_HANDLE_T hUsb, void *data, uint32_t event)
{
	switch (event) {
	case USB_EVT_OUT_NAK:
		if (g_ep0RxBusy) {
			// we already queued the buffer so ignore this NAK event. 
			return LPC_OK;
		}
		else {
			// Mark EP0_RX buffer as busy and allow base handler to queue the buffer. 
			g_ep0RxBusy = 1;
		}
		break;

	case USB_EVT_SETUP:	// reset the flag when new setup sequence starts 
	case USB_EVT_OUT:
		// we received the packet so clear the flag. 
		g_ep0RxBusy = 0;
		break;
	}
	return g_Ep0BaseHdlr(hUsb, data, event);
}
*/
ErrorCode_t USB_Interface_Event (USBD_HANDLE_T hUsb)
{
	return LPC_OK;
}

ErrorCode_t UsbdRom_Event_Stub (USBD_HANDLE_T hUsb)
{
	return LPC_OK;
}

ErrorCode_t USB_Configure_Event (USBD_HANDLE_T hUsb)
{
	USB_CORE_CTRL_T* pCtrl = (USB_CORE_CTRL_T*) hUsb;
	uint8_t epnum = 0;

	if((pCtrl->config_value)&&(epnum!=0)) 
		USBD_API->hw->WriteEP(hUsb, (epnum + 0x80), usb_RomDriver_buffer, 1);
	return LPC_OK;
}

ErrorCode_t HID_GetReport(USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t* plength)
{
	// ReportID = SetupPacket.wValue.WB.L;

	switch (pSetup->wValue.WB.H) {

	case HID_REPORT_INPUT:	
		break;

	case HID_REPORT_OUTPUT:
		return (ERR_USBD_STALL);          /* Not Supported */

	case HID_REPORT_FEATURE:
		return (ERR_USBD_STALL);          /* Not Supported */
	}

	return (LPC_OK);
}

ErrorCode_t HID_SetReport( USBD_HANDLE_T hHid, USB_SETUP_PACKET* pSetup, uint8_t** pBuffer, uint16_t length)
{
	return (LPC_OK);
}

ErrorCode_t HID_EpIn_Hdlr(USBD_HANDLE_T hUsb, void* data, uint32_t event)
{
	switch (event) {

	case USB_EVT_IN:
		GPIO1_PBYTE8 = 1;
		tx_busy = 0;
		break;
	}

	return LPC_OK;
}

void UsbdRom_Init(USBD_API_INIT_PARAM_T* usb_param) 
{
	ErrorCode_t ret;

	usb_param->usb_reg_base = LPC_USB_BASE;
	usb_param->max_num_ep = ENDPOINT_TOTAL_ENDPOINTS;
	usb_param->mem_base = (uint32_t) usb_RomDriver_buffer; 
	usb_param->mem_size = ROMDRIVER_MEM_SIZE;
	usb_param->USB_Configure_Event = USB_Configure_Event;

	USB_CORE_DESCS_T DeviceDes = {NULL};

	/* add custom Interface Event */
	if (USB_Interface_Event != UsbdRom_Event_Stub) {
		usb_param->USB_Interface_Event = USB_Interface_Event;
	}

	DeviceDes.device_desc = (uint8_t*) &DeviceDesc;
	DeviceDes.high_speed_desc = (uint8_t*) &ConfigDesc;
	DeviceDes.full_speed_desc = (uint8_t*) &ConfigDesc;
	DeviceDes.string_desc = (uint8_t*) StringDescriptor;
	DeviceDes.device_qualifier = 0;

	ret = USBD_API->hw->Init(&UsbHandle, &DeviceDes, usb_param);

	// trap failed initialization

	while(ret != LPC_OK);
}

ErrorCode_t usb_hid_init(USBD_API_INIT_PARAM_T* usb_param)
{
	USB_INTERFACE_DESCRIPTOR* pIntfDesc;
	USBD_HID_INIT_PARAM_T hid_param;
	USB_HID_REPORT_T reports_data;
	ErrorCode_t ret = LPC_OK;

	memset((void*)&hid_param, 0, sizeof(USBD_HID_INIT_PARAM_T));

	pIntfDesc = (USB_INTERFACE_DESCRIPTOR *) &ConfigDesc[9];

	if ((pIntfDesc == 0) || (pIntfDesc->bInterfaceClass != USB_DEVICE_CLASS_HUMAN_INTERFACE))
		return ERR_FAILED;

	hid_param.mem_base = usb_param->mem_base;
	hid_param.mem_size = usb_param->mem_size; 
	hid_param.intf_desc = (uint8_t*)pIntfDesc;		
	hid_param.max_reports = 1;

	reports_data.desc = ReportDesc;
	reports_data.len = sizeof(ReportDesc);

	reports_data.idle_time = 0;

	// User defined functions

	hid_param.HID_GetReport = HID_GetReport;
	hid_param.HID_SetReport = HID_SetReport;
	hid_param.HID_EpIn_Hdlr = HID_EpIn_Hdlr;
	hid_param.report_data  = &reports_data;

	ret = USBD_API->hid->init(UsbHandle, &hid_param);

	return ret;
}

void USB_IRQHandler(void)
{
	USBD_API->hw->ISR(UsbHandle);
}

void EnableEP(USBD_HANDLE_T hUsb, uint32_t EPnum)
{
	USBD_API->hw->EnableEP(hUsb, EPnum);
}

void DisableEP(USBD_HANDLE_T hUsb, uint32_t EPnum)
{
	USBD_API->hw->EnableEP(hUsb, EPnum);
}

void USBconnect(void)
{
	USBD_API->hw->Connect(UsbHandle,1);
}

void USBdisconnect(void)
{
	USBD_API->hw->Connect(UsbHandle,0);
}
